#include <ncdc/cmds.h>
#include <ncdc/ncdc.h>

static bool
ncdc_cmd_friends_list(ncdc_mainwindow_t n, size_t ac, wchar_t **av)
{
    bool ret = false;
    size_t i = 0;
    char c = ' ';

    ret = dc_api_get_friends(api, current_account);
    if (!ret) {
        LOG(n, L"friends: list: failed to fetch your friends");
        return false;
    }

    LOG(n, L"/FRIENDS list");
    for (i = 0; i < dc_account_friends_size(current_account); i++) {
        dc_account_t acc = dc_account_nthfriend(current_account, i);
        switch (dc_account_friend_state(acc)) {
        case FRIEND_STATE_PENDING: c = 'P'; break;
        default: c = ' '; break;
        }
        LOG(n, L" %c %s", c, dc_account_fullname(acc));
    }
    LOG(n, L"End of /FRIENDS list");

    return true;
}

static bool
ncdc_cmd_friends_add(ncdc_mainwindow_t n, size_t ac, wchar_t **av)
{
    char *name = NULL;
    dc_account_t friend = NULL;
    bool ret = false;

    if (ac <= 1) {
        return false;
    }

    name = w_convert(av[1]);
    return_if_true(name == NULL, false);

    friend = dc_account_from_fullname(name);
    if (friend == NULL) {
        LOG(n, L"friends: add: invalid username given, use the full ID");
        goto cleanup;
    }

    if (!dc_api_add_friend(api, current_account, friend)) {
        LOG(n, L"friends: add: failed to add friend, Vulkan would be sad");
        goto cleanup;
    }

    LOG(n, L"friends: add: request for friendship sent to %s", name);
    ret = true;

cleanup:

    dc_unref(friend);
    free(name);

    return ret;
}

static bool
ncdc_cmd_friends_remove(ncdc_mainwindow_t n, size_t ac, wchar_t **av)
{
    char *name = NULL;
    dc_account_t friend = NULL;
    bool ret = false;
    size_t i = 0;

    if (ac <= 1) {
        return false;
    }

    name = w_convert(av[1]);
    return_if_true(name == NULL, false);

    for (i = 0; i < dc_account_friends_size(current_account); i++) {
        dc_account_t cur = dc_account_nthfriend(current_account, i);
        if (strcmp(dc_account_fullname(cur), name) == 0) {
            friend = cur;
            break;
        }
    }

    if (friend == NULL) {
        LOG(n, L"friends: remove: no such friend in friend's list");
        goto cleanup;
    }

    if (!dc_api_remove_friend(api, current_account, friend)) {
        LOG(n, L"friends: remove: failed to remove friend");
        goto cleanup;
    }

    LOG(n, L"friends: remove: friend %s removed", name);
    ret = true;

cleanup:

    free(name);

    return ret;
}

static bool
ncdc_cmd_friends_accept(ncdc_mainwindow_t n, size_t ac, wchar_t **av)
{
    char *name = NULL;
    dc_account_t friend = NULL;
    bool ret = false;
    size_t i = 0;

    if (ac <= 1) {
        return false;
    }

    name = w_convert(av[1]);
    return_if_true(name == NULL, false);

    for (i = 0; i < dc_account_friends_size(current_account); i++) {
        dc_account_t cur = dc_account_nthfriend(current_account, i);
        if (strcmp(dc_account_fullname(cur), name) == 0 &&
            dc_account_friend_state(cur) == FRIEND_STATE_PENDING) {
            friend = cur;
            break;
        }
    }

    if (friend == NULL) {
        LOG(n, L"friends: accept: no such pending friend on the list");
        goto cleanup;
    }

    if (!dc_api_accept_friend(api, current_account, friend)) {
        LOG(n, L"friends: accept: failed to accept friend");
        goto cleanup;
    }

    LOG(n, L"friends: accept: friend %s accepted", name);
    ret = true;

cleanup:

    free(name);

    return ret;
}

static ncdc_commands_t subcmds[] = {
    { L"accept", ncdc_cmd_friends_accept },
    { L"add",    ncdc_cmd_friends_add },
    { L"list",   ncdc_cmd_friends_list },
    { L"remove", ncdc_cmd_friends_remove },
    { NULL,     NULL }
};

bool ncdc_cmd_friends(ncdc_mainwindow_t n, size_t ac, wchar_t **av)
{
    wchar_t *subcmd = NULL;
    ncdc_commands_t *it = NULL;

    if (current_account == NULL ||
        !dc_account_has_token(current_account)) {
        LOG(n, L"friends: not logged in");
        return false;
    }

    if (ac <= 1) {
        return ncdc_cmd_friends_list(n, ac, av);
    }

    subcmd = av[1];

    --ac;
    ++av;

    if ((it = ncdc_find_cmd(subcmds, subcmd)) == NULL) {
        LOG(n, L"friends: no such subcommand \"%ls\"", subcmd);
        return false;
    }

    return it->handler(n, ac, av);
}
